/*
 * Copyright 1999-2006 University of Chicago
 * 
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.globus.common;

/** 
 * This class contains version information of the JGlobus module.
 * The version number is composed as MAJOR.MINOR.PATCH.
 */
public class Version {
    
    /** The major release number */
    public static final int MAJOR = 1;
    
    /** The minor release number */
    public static final int MINOR = 4;
    
    /** The patchlevel of the current release */
    public static final int PATCH = 0;
    
    /** 
     * Returns the current version as string in the form MAJOR.MINOR.PATCH.
     */
    public static String getVersion() {
        return getMajor() + "." + getMinor() + "." + getPatch();
    }
    
    /**
     * Returns the major release number.
     * 
     * @return the major release
     */
    public static int getMajor() {
        return MAJOR;
    }
    
    /**
     * Returns the minor release number.
     * 
     * @return the minor release number
     */
    public static int getMinor() {
        return MINOR;
    }
    
    /** 
     * Returns the patch level.
     * 
     * @return the patch level
     */
    public static int getPatch() {
        return PATCH;
    }
    
    /** 
     * Returns the version for the Java CoG Kit as a readble string.
     * 
     * @param args 
     */
    public static void main(String [] args) {
        System.out.println("Java CoG version: " + getVersion());
    }
    
}
