/*
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Library General Public License as published by the Free
 * Software Foundation; either version 2 of the License, or (at your option) any
 * later version.
 *
 * This library is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU Library General Public License for more
 * details.
 *
 * You should have received a copy of the GNU Library General Public License
 * along with this program (see the file COPYING.LIB for more details); if not,
 * write to the Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA
 * 02139, USA.
 */
package org.dcache.chimera.nfs.v4;

import org.dcache.chimera.nfs.v4.xdr.server_owner4;


/**
 * Interface to provide server unique information used
 * during EXCHANGE_ID operation.
 *
 * The server scope allows a group of servers to indicate to clients that a set
 * of servers sharing the same server scope value has arranged to use compatible
 * values of otherwise opaque identifiers.  Thus, the identifiers generated by
 * one server of that set may be presented to another of that same scope.
 *
 * See rfc5661 section 2.10.4 for details.
 */
public interface ServerIdProvider {

    server_owner4 getOwner();

    byte[] getScope();
}