package org.dcache.webadmin.view.pages.poollist;

import diskCacheV111.pools.PoolV2Mode;
import java.util.List;
import java.util.ArrayList;
import org.apache.wicket.authorization.strategies.role.metadata.MetaDataRoleAuthorizationStrategy;
import org.apache.wicket.markup.html.form.Button;
import org.apache.wicket.markup.html.form.ChoiceRenderer;
import org.apache.wicket.markup.html.form.DropDownChoice;
import org.apache.wicket.markup.html.form.Form;
import org.apache.wicket.markup.html.panel.FeedbackPanel;
import org.apache.wicket.model.PropertyModel;
import org.apache.wicket.model.StringResourceModel;
import org.dcache.webadmin.controller.PoolSpaceService;
import org.dcache.webadmin.controller.exceptions.PoolSpaceServiceException;
import org.dcache.webadmin.view.beans.PoolSpaceBean;
import org.dcache.webadmin.view.beans.SelectOption;
import org.dcache.webadmin.view.pages.basepage.BasePage;
import org.dcache.webadmin.view.panels.poollist.PoolListPanel;
import org.dcache.webadmin.view.util.Role;
import org.slf4j.LoggerFactory;
import org.slf4j.Logger;

/**
 * The PoolUsage Webpage
 * @author jans
 */
public class PoolList extends BasePage {

    private static final int DEFAULT_DROP_DOWN_CHOICE = 0;
    private List<PoolSpaceBean> _poolBeans;
    private SelectOption _selectedOption;
    private static final Logger _log = LoggerFactory.getLogger(PoolList.class);

    public PoolList() {
        Form poolUsageForm = new PoolUsageForm("poolUsageForm");
        poolUsageForm.add(createPoolModeDropDown("mode"));
        poolUsageForm.add(new FeedbackPanel("feedback"));
        getPoolsAction();
        PoolListPanel poolListPanel = new PoolListPanel("poolListPanel",
                new PropertyModel(this, "_poolBeans"), true);
        poolUsageForm.add(poolListPanel);
        add(poolUsageForm);
    }

    private DropDownChoice createPoolModeDropDown(String id) {
        List<SelectOption> dropDownChoices = getDropDownModes();
        DropDownChoice dropDownChoice = new DropDownChoice(id,
                new PropertyModel(this, "_selectedOption"),
                dropDownChoices, new ChoiceRenderer<SelectOption>("value"));
        setDefaultChoice(dropDownChoices);
        MetaDataRoleAuthorizationStrategy.authorize(dropDownChoice,
                RENDER, Role.ADMIN);
        return dropDownChoice;
    }

    private void setDefaultChoice(List<SelectOption> dropDownChoices) {
        _selectedOption = dropDownChoices.get(DEFAULT_DROP_DOWN_CHOICE);
    }

    private List<SelectOption> getDropDownModes() {
        List<SelectOption> list = new ArrayList<SelectOption>(3);
        list.add(new SelectOption(PoolV2Mode.ENABLED, new StringResourceModel(
                "mode.ENABLED", this, null).getString()));
        list.add(new SelectOption(PoolV2Mode.DISABLED_STRICT, new StringResourceModel(
                "mode.DISABLED_STRICT", this, null).getString()));
        list.add(new SelectOption(PoolV2Mode.DISABLED_RDONLY, new StringResourceModel(
                "mode.DISABLED_RDONLY", this, null).getString()));
        return list;
    }

    private PoolSpaceService getPoolSpaceService() {
        return getWebadminApplication().getPoolSpaceService();
    }

    private void getPoolsAction() {
        try {
            _log.debug("getPoolListAction called");
            this._poolBeans = getPoolSpaceService().getPoolBeans();
        } catch (PoolSpaceServiceException ex) {
            this.error(getStringResource("error.getPoolsFailed") + ex.getMessage());
            _log.debug("getPoolListAction failed {}", ex.getMessage());
            this._poolBeans = null;
        }
    }

    private class PoolUsageForm extends Form {

        public PoolUsageForm(String id) {
            super(id);
            Button button = new Button("submit");
            MetaDataRoleAuthorizationStrategy.authorize(button, RENDER, Role.ADMIN);
            _log.debug("submit isEnabled : {}", String.valueOf(button.isEnabled()));
            this.add(button);
        }

        @Override
        protected void onSubmit() {
            _log.debug("button pressed");
            if (_poolBeans != null && _selectedOption != null) {
                try {
                    _log.debug("selected: {}", _selectedOption.getValue());
                    PoolV2Mode poolMode = new PoolV2Mode(_selectedOption.getKey());
                    getPoolSpaceService().changePoolMode(_poolBeans, poolMode,
                            getWebadminSession().getUserName());
                    getPoolsAction();
                } catch (PoolSpaceServiceException ex) {
                    _log.error("something went wrong with enable/disable");
                    this.error(getStringResource("error.changePoolModeFailed") + ex.getMessage());
                }
            }
        }
    }
}
